@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunffb/ffb_driver.c,v 1.15 2005/02/18 02:55:09 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunffb/ffb_driver.c,v 1.19 2007/01/23 18:03:08 tsi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -44,8 +44,8 @@ static const OptionInfoRec * FFBAvailableOptions(int chipid, int busid);
 static void	FFBIdentify(int flags);
 static Bool	FFBProbe(DriverPtr drv, int flags);
 static Bool	FFBPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool	FFBScreenInit(int Index, ScreenPtr pScreen, int argc,
-			      char **argv);
+static Bool	FFBScreenInit(int Index, ScreenPtr pScreen,
+			      const int argc, const char **argv);
 static Bool	FFBEnterVT(int scrnIndex, int flags);
 static void	FFBLeaveVT(int scrnIndex, int flags);
 static Bool	FFBCloseScreen(int scrnIndex, ScreenPtr pScreen);
@@ -123,7 +123,7 @@ static XF86ModuleVersionInfo sunffbVersRec =
 XF86ModuleData sunffbModuleData = { &sunffbVersRec, ffbSetup, NULL };
 
 static pointer
-ffbSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+ffbSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -664,7 +664,8 @@ CreatorUnaccelWidInit(ScreenPtr pScreen)
 /* This gets called at the start of each server generation */
 
 static Bool
-FFBScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+FFBScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn;
     FFBPtr pFfb;
@@ -808,9 +809,6 @@ FFBScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	    xf86Msg(X_INFO, "%s: ... AFB firmware is loaded\n", psdp->device);
     }
 
-    /* Darken the screen for aesthetic reasons and set the viewport */
-    FFBSaveScreen(pScreen, SCREEN_SAVER_ON);
-
     if (pFfb->NoAccel == TRUE) {
 	    if (!CreatorUnaccelWidInit(pScreen))
 		    return FALSE;
@@ -961,9 +959,6 @@ FFBScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);
     }
 
-    /* unblank the screen */
-    FFBSaveScreen(pScreen, SCREEN_SAVER_OFF);
-
     /* Done */
     return TRUE;
 }
@@ -1055,6 +1050,8 @@ FFBCloseScreen(int scrnIndex, ScreenPtr pScreen)
 	    FFBDRICloseScreen(pScreen);
 #endif
 
+    FFBDacCursorEnableDisable(pFfb, 0);
+
     /* Restore kernel ramdac state before we unmap registers. */
     FFBDacFini(pFfb);
 
@@ -1106,13 +1103,12 @@ FFBValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 static Bool
 FFBSaveScreen(ScreenPtr pScreen, int mode)
     /* This function blanks the screen when mode=SCREEN_SAVER_ON and
-       unblanks it when mode=SCREEN_SAVER_OFF.  It is used internally in the
-       FFBScreenInit code `for aesthetic reasons,' and it is used for
+       unblanks it when mode=SCREEN_SAVER_OFF.  It is used for
        blanking if you set "xset s on s blank."  The work (such as it is) is
        done in "ffb_dac.c" `for aesthetic reasons.'
     */
 {
-    return FFBDacSaveScreen(GET_FFB_FROM_SCREEN(pScreen), mode);
+    return FFBDacSaveScreen(xf86Screens[pScreen->myNum], mode);
 }
 
 /*
