@@ -27,7 +27,7 @@ other dealings in this Software without prior written authorization
 from the X Consortium.
 
 */
-/* $XFree86: xc/programs/xman/man.c,v 1.12 2006/01/09 15:01:37 dawes Exp $ */
+/* $XFree86: xc/programs/xman/man.c,v 1.13 2006/12/30 00:58:10 tsi Exp $ */
 
 
 #include "globals.h"
@@ -50,6 +50,10 @@ from the X Consortium.
 #endif
 #endif
 
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+#include <sys/utsname.h>
+#endif
+
 #ifdef DEBUG
 static char error_buf[BUFSIZ];		/* The buffer for error messages. */
 #endif /* DEBUG */
@@ -396,11 +400,17 @@ int flags)
 static void
 AddToCurrentSection(Manual * local_manual, char * path)
 {
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+  struct utsname uts;
+#endif
   char temp_path[BUFSIZ];
 
 #if defined(__OpenBSD__) || defined(__NetBSD__)
-  sprintf(temp_path, "%s/%s", path, MACHINE);
+  /* XXX: No way to deal w. error here so just move on. */
+  if (uname(&uts) == 0) {
+    sprintf(temp_path, "%s/%s", path, uts.machine);
   ReadCurrentSection(local_manual, temp_path);
+  }
 #endif
   ReadCurrentSection(local_manual, path);
   sprintf(temp_path, "%s.%s", path, COMPRESSION_EXTENSION);
